@@ -56,7 +56,7 @@ from .subscription import (
     Subscription,
 )
 
-__version__ = '2.1.0'
+__version__ = '2.1.2'
 __lang__ = 'python3'
 _logger = logging.getLogger(__name__)
 PROTOCOL = 1
@@ -188,9 +188,10 @@ class Client:
         # max pending size is the maximum size of the data that can be buffered.
         self._max_pending_size: int = 0
 
-        self._flush_queue: Optional["asyncio.Queue[None]"] = None
+        self._flush_queue: Optional["asyncio.Queue[asyncio.Future[Any]]"
+                                    ] = None
         self._flusher_task: Optional[asyncio.Task] = None
-        self._flush_timeout: float = 0
+        self._flush_timeout: Optional[float] = 0
         self._hdr_parser: BytesParser = BytesParser()
 
         # New style request/response
@@ -810,7 +811,10 @@ class Client:
         If a callback isn't provided, messages can be retrieved via an
         asynchronous iterator on the returned subscription object.
         """
-        if not subject:
+        if not subject or (' ' in subject):
+            raise errors.BadSubjectError
+
+        if queue and (' ' in queue):
             raise errors.BadSubjectError
 
         if self.is_closed:
@@ -933,8 +937,15 @@ class Client:
             msg = await asyncio.wait_for(future, timeout)
             return msg
         except asyncio.TimeoutError:
-            # Double check that the token is there already.
-            self._resp_map.pop(token.decode())
+            try:
+                # Double check that the token is there already.
+                self._resp_map.pop(token.decode())
+            except KeyError:
+                await self._error_cb(
+                    errors.
+                    Error(f"nats: missing response token '{token.decode()}'")
+                )
+
             future.cancel()
             raise errors.TimeoutError
 
@@ -1095,10 +1106,10 @@ class Client:
     async def _flush_pending(
         self,
         force_flush: bool = False,
-    ) -> None:
+    ) -> Any:
         assert self._flush_queue, "Client.connect must be called first"
         try:
-            future = asyncio.Future()
+            future: "asyncio.Future" = asyncio.Future()
             if not self.is_connected:
                 future.set_result(None)
                 return future
@@ -1733,7 +1744,7 @@ class Client:
 
         if 'tls_required' in self._server_info and self._server_info[
                 'tls_required']:
-            ssl_context: Optional[ssl.SSLContext]
+            ssl_context: Optional[ssl.SSLContext] = None
             if "tls" in self.options:
                 ssl_context = self.options.get('tls')
             elif self._current_server.uri.scheme == 'tls':
