@@ -41,6 +41,7 @@ class StatusCode(str, Enum):
     SERVICE_UNAVAILABLE = "503"
     NO_MESSAGES = "404"
     REQUEST_TIMEOUT = "408"
+    CONFLICT = "409"
     CONTROL_MESSAGE = "100"
 
 
@@ -128,7 +129,7 @@ class PubAck(Base):
 class Placement(Base):
     """Placement directives to consider when placing replicas of this stream"""
 
-    cluster: str
+    cluster: Optional[str] = None
     tags: Optional[List[str]] = None
 
 
